function Results = FBSM_minisys(Z_ser,S_in,U_in,connections,maxIter,convEps,doPlot)
% Forward Backward Sweep Method (FBSM) solver for the minisystem wit 4
% buses
%
% Results = FBSM_minisys(Z_in,S_in,U_in,connections,MAX_ITER,convEps,doPlot)
%
% Inputs:
%    Z_in        = Impedance matrix for all connections (NxN).
%    S_in        = Power consumption (3-phase) at load buses (others 0) (Nx1).
%    U_in        = Voltage guess for each bus (specified voltage at slack bus) (Nx1).
%    connections = Matrix decribing connections between buses (N-1x2).
%    maxIter     = Maximum number of iterations. Default value from Settings.
%    convEps     = Convergence criteria. Default value from Settings.
%    doPlot      = Create plots (1x1 logical). Default value from Settings.
%
% Outputs:
%    Results.S_out   = Power calculation (per bus)
%    Results.S_loss  = Power loss calculation (per connection)
%    Results.U_out   = Voltage calculation (per bus)
%    Results.U_loss  = Voltage loss calculation (per connection)
%    Results.I_out   = Current calculation (per connection)
%    Results.nIters  = Number of iterations

global Settings;
if ~exist('maxIter','var'),  maxIter  = Settings.defaultMaxIter;   end    % Default value for maximum number of iterations
if ~exist('convEps','var'),  convEps  = Settings.defaultConvEps;   end    % Default convergence limit
if ~exist('doPlot','var'),   doPlot   = Settings.defaultConvPlots; end    % Default value for plot creation

% Inputs
S_calc(:,1) = S_in;     % Input powers (loads) at iteration 1
U_calc(:,1) = U_in;     % Input voltages (guesses) at iteration 1

% Current matrix preallocations (excluding inputs)
I_calc(:,1) = zeros(3,1);

% Power matrix preallocations (excluding inputs)
S_conn(:,1) = zeros(3,1);
S_loss(:,1) = zeros(3,1);

% Voltage matrix preallocations (excluding inputs)
U_delta(:,1) = zeros(3,1);

iter = 2;               % First calculation is iteration 2

while iter<=maxIter
    
    if iter == maxIter
        warning('No convergence before maximum number of iterations was reached.') 
    end
 
    % Backward sweep to calculate powers and currents
    S_calc(:,iter) = S_in; %Given in problem
    S_loss(:,iter) = zeros(3,1);
    S_conn(:,iter) = zeros(3,1);
    I_calc(:,iter) = zeros(3,1);
    
    I_calc(2,iter) = conj(S_calc(3,iter)/(sqrt(3)*U_calc(3,iter-1)));
    I_calc(3,iter) = conj(S_calc(4,iter)/(sqrt(3)*U_calc(4,iter-1)));
    
    %Beräkna strömmen i kabel 1.
    I_calc(1,iter) = I_calc(2,iter)+I_calc(3,iter);
    
    %Beräkna skenbara förlusterna i respektive kabel
    S_loss(1,iter) = 3*I_calc(1,iter)*conj(I_calc(1,iter))...
                    *Z_ser(1,2);
    S_loss(2,iter) = 3*I_calc(2,iter)*conj(I_calc(2,iter))...
                    *Z_ser(2,3);	
    S_loss(3,iter) = ???
    
    
    %Beräkna skenbara effekten som matas in i respektive kabel
    S_conn(2,iter) = S_calc(3,iter)+S_loss(2,iter);
    S_conn(3,iter) = S_calc(4,iter)+S_loss(3,iter); 
    S_calc(2,iter) = S_conn(2,iter)+S_conn(3,iter);
    S_conn(1,iter) = ???; %Beräknar S i kabel 1
    S_calc(1,iter) = ???;  % Beräknar S i buss 1
       
    
    %Initialixe forward sweep
    U_calc(:,iter)=U_in;                                % Input voltages (guesses)
    U_delta(:,iter) = zeros(3,1);                       % Matrix preallocation
    
    % Forward sweep to calculate voltages
    U_delta(1,iter) = -sqrt(3)*(I_calc(1,iter))*Z_ser(1,2);  % Spänningsfall i ledning 1
    U_calc(2,iter)  = U_calc(1,iter)+U_delta(1,iter); % Spänning i buss 2
    U_delta(2,iter) = -sqrt(3)*(I_calc(2,iter))*Z_ser(2,3);  % Spänningsfall i ledning 2
    U_calc(3,iter)  = U_calc(2,iter)+U_delta(2,iter);  % Spänning i buss 3
    U_delta(3,iter) = ???   % Spänningsfall i ledning 3
    U_calc(4,iter)  = ??? % Spänning i buss 4.
    
        
    % Convergence criteria
    if iter > 2
        powerConvCrit   = max(abs(S_calc(:,iter-1) - S_calc(:,iter)));      % Power convergence criterion
        voltageConvCrit = max(abs(U_calc(:,iter-1) - U_calc(:,iter)));      % Voltage convergence criterion
        currentConvCrit = max(abs(I_calc(:,iter-1) - I_calc(:,iter)));      % Current convergence criterion
        
        if powerConvCrit < convEps && voltageConvCrit < convEps && currentConvCrit < convEps
            break
        end
    end
  
    iter = iter+1;              % Go to next iteration
end

% Plot convergence
if doPlot
    legendLabelsU=[repmat('U_{',length(Z_ser),1) num2str(transpose(1:length(Z_ser))) repmat('}',length(Z_ser),1)];
    legendLabelsP=[repmat('P_{',length(Z_ser),1) num2str(transpose(1:length(Z_ser))) repmat('}',length(Z_ser),1)];
    legendLabelsQ=[repmat('Q_{',length(Z_ser),1) num2str(transpose(1:length(Z_ser))) repmat('}',length(Z_ser),1)];
    legendLabelsI=[repmat('I_{',size(connections,1),1) num2str(transpose(1:size(connections,1))) repmat('}',size(connections,1),1)];
    figure;
    plot(abs(U_calc'));
    title('Voltage convergence');
    xlabel('Number of iterations');
    ylabel('Voltage');
    legend(legendLabelsU);
    figure;
    plot(real(S_calc'));
    title('Active power convergence');
    xlabel('Number of iterations');
    ylabel('Active power');
    legend(legendLabelsP);
    figure;
    plot(imag(S_calc'));
    title('Reactive power convergence');
    xlabel('Number of iterations');
    ylabel('Reactive power');
    legend(legendLabelsQ);
    figure;
    plot(abs(I_calc'));
    title('Current convergence');
    xlabel('Number of iterations');
    ylabel('Current');
    legend(legendLabelsI);
end

% Output
Results.S_out   = S_calc(:,end);      % Power calculation (per bus)
Results.S_loss  = S_loss(:,end);      % Power loss calculation (per connection)
Results.U_out   = U_calc(:,end);      % Voltage calculation (per bus)
Results.U_delta = U_delta(:,end);     % Voltage loss calculation (per connection)
Results.I_out   = I_calc(:,end);      % Current calculation (per connection)
Results.nIters  = iter;               % Number of iterations

end